@@ -54,9 +54,9 @@ class PackageRemovalDeniedException(Exception):
         self.package = package
 
         msg = _(
-            f'The package {self.package} is marked for removal '
+            'The package {} is marked for removal '
             'but it is in the removal deny list.'
-        )
+        ).format(self.package)
 
         # If additional details can be given to the user explaining the
         # situtation, provide them here.
@@ -64,9 +64,9 @@ class PackageRemovalDeniedException(Exception):
             msg += _(
                 '\n\nTo prevent data loss, postgresql packages are not removed '
                 'automatically during the upgrade. If you are certain you '
-                f'no longer need {self.package}, you can manually remove it '
+                'no longer need {}, you can manually remove it '
                 'and try the upgrade again.'
-            )
+            ).format(self.package)
 
         super().__init__(msg)
 
@@ -521,47 +521,6 @@ class MyCache(apt.Cache):
         if not self.partialUpgrade:
             self.quirks.PostDistUpgradeCache()
 
-    def checkForNvidia(self):
-        """
-        this checks for nvidia hardware and checks what driver is needed
-        """
-        logging.debug("nvidiaUpdate()")
-        # if the free drivers would give us a equally hard time, we would
-        # never be able to release
-        try:
-            from NvidiaDetector.nvidiadetector import NvidiaDetection
-        except (ImportError, SyntaxError) as e:
-            # SyntaxError is temporary until the port of NvidiaDetector to
-            # Python 3 is in the archive.
-            logging.error("NvidiaDetector can not be imported %s" % e)
-            return False
-        try:
-            # get new detection module and use the modalises files
-            # from within the release-upgrader
-            nv = NvidiaDetection(obsolete="./ubuntu-drivers-obsolete.pkgs")
-            #nv = NvidiaDetection()
-            # check if a binary driver is installed now
-            for oldDriver in nv.oldPackages:
-                if oldDriver in self and self[oldDriver].is_installed:
-                    self.mark_remove(oldDriver, "old nvidia driver")
-                    break
-            else:
-                logging.info("no old nvidia driver installed, installing no new")
-                return False
-            # check which one to use
-            driver = nv.selectDriver()
-            logging.debug("nv.selectDriver() returned '%s'" % driver)
-            if not driver in self:
-                logging.warning("no '%s' found" % driver)
-                return False
-            if not (self[driver].marked_install or self[driver].marked_upgrade):
-                self[driver].mark_install()
-                logging.info("installing %s as suggested by NvidiaDetector" % driver)
-                return True
-        except Exception as e:
-            logging.error("NvidiaDetection returned a error: %s" % e)
-        return False
-
     def checkForKernel(self):
         """ check for the running kernel and try to ensure that we have
             an updated version
@@ -643,9 +602,6 @@ class MyCache(apt.Cache):
             else:
                 self.checkForKernel()
 
-            # check for nvidia stuff
-            self.checkForNvidia()
-
             # and if we have some special rules
             self.post_upgrade_rule()
 
@@ -1046,7 +1002,7 @@ class MyCache(apt.Cache):
                 continue
 
             if self.downloadable(pkg):
-                for (pkg_file, _) in pkg.candidate._cand.file_list:
+                for (pkg_file, _index) in pkg.candidate._cand.file_list:
                     # Only consider configured apt sources
                     if pkg_file.not_source:
                         continue
