@@ -2,7 +2,7 @@
 
 set -e
 
-DIST=$(lsb_release -c -s)
+DIST=$(. /etc/os-release && echo "$VERSION_CODENAME")
 
 # cleanup
 echo "Cleaning up"
