@@ -34,7 +34,11 @@ echo "copying apt_btrfs_snapshot.py for the upgrader tarball"
 cp /usr/lib/python3/dist-packages/apt_btrfs_snapshot.py DistUpgrade
 
 # update the translations
-cd po; make update-po; cd ../
+if [ -n "${PREBUILD_SKIP_TRANSLATIONS:-""}" ]; then
+    echo "skipping translation step"
+else
+    cd po; make update-po; cd ../
+fi
 
 # (auto) generate the required html
 if [ ! -x /usr/bin/parsewiki ]; then
