@@ -582,6 +582,129 @@ max_framebuffers=2
             q._test_and_fail_on_rva23()
             self.assertEqual(1, len(mock_controller.abort.mock_calls))
 
+    def test_test_and_fail_on_pi4_old_eeprom(self):
+        from pathlib import Path
+        with tempfile.TemporaryDirectory() as dt_dir:
+            dt = Path(dt_dir)
+            bootloader = dt / 'chosen' / 'bootloader'
+            bootloader.mkdir(parents=True)
+
+            mock_controller = mock.Mock()
+            mock_controller.arch = 'arm64'
+
+            q = DistUpgradeQuirks(mock_controller, mock.Mock())
+
+            # No d-t/compatible (no abort)
+            mock_controller.abort.reset_mock()
+            q._test_and_fail_on_pi4_old_eeprom(dt_dir)
+            self.assertFalse(mock_controller.abort.called)
+            # d-t/compatible not rpi (no abort)
+            mock_controller.abort.reset_mock()
+            (dt / 'compatible').write_bytes(b'foo,bar\0foo,baz\0')
+            q._test_and_fail_on_pi4_old_eeprom(dt_dir)
+            self.assertFalse(mock_controller.abort.called)
+            # d-t/compatible is rpi, new build-timestamp (no abort)
+            mock_controller.abort.reset_mock()
+            (dt / 'compatible').write_bytes(
+                b'raspberrypi,4-model-b\0brcm,bcm2711\0')
+            (bootloader / 'build-timestamp').write_bytes(
+                b'\x66\xf1\x67\x00')  # 1727096576 (2024-09-23)
+            q._test_and_fail_on_pi4_old_eeprom(dt_dir)
+            self.assertFalse(mock_controller.abort.called)
+            # d-t/compatible is rpi, old build-timestamp (abort)
+            mock_controller.abort.reset_mock()
+            (bootloader / 'build-timestamp').write_bytes(
+                b'\x5f\x50\xdd\x7f')  # 1599135103 (2020-09-03)
+            q._test_and_fail_on_pi4_old_eeprom(dt_dir)
+            self.assertTrue(mock_controller.abort.called)
+            # d-t/compatible is rpi, no build-timestamp (abort)
+            mock_controller.abort.reset_mock()
+            (bootloader / 'build-timestamp').unlink()
+            q._test_and_fail_on_pi4_old_eeprom(dt_dir)
+            self.assertTrue(mock_controller.abort.called)
+
+    def test_and_fail_without_cgroupv2(self):
+        mock_controller = mock.Mock()
+        q = DistUpgradeQuirks(mock_controller, mock.Mock())
+
+        m = mock.mock_open(read_data=(
+            'sysfs /sys sysfs '
+            'rw,nosuid,nodev,noexec,relatime 0 0\n'
+            'proc /proc proc '
+            'rw,nosuid,nodev,noexec,relatime 0 0\n'
+            'udev /dev devtmpfs '
+            'rw,nosuid,relatime,size=16029572k,mode=755,inode64 0 0\n'
+            'devpts /dev/pts devpts '
+            'rw,nosuid,noexec,relatime,gid=5,mode=620,ptmxmode=000 0 0\n'
+            'tmpfs /run tmpfs '
+            'rw,nosuid,nodev,noexec,relatime,mode=755,inode64 0 0\n'
+            'securityfs /sys/kernel/security securityfs '
+            'rw,nosuid,nodev,noexec,relatime 0 0\n'
+            'tmpfs /dev/shm tmpfs '
+            'rw,nosuid,nodev,inode64 0 0\n'
+            'tmpfs /run/lock tmpfs '
+            'rw,nosuid,nodev,noexec,relatime,size=5120k,inode64 0 0\n'
+            'cgroup2 /sys/fs/cgroup cgroup2 '
+            'rw,nosuid,nodev,noexec,relatime 0 0\n'
+            'pstore /sys/fs/pstore pstore '
+            'rw,nosuid,nodev,noexec,relatime 0 0\n'
+        ))
+        with mock.patch('DistUpgrade.DistUpgradeQuirks.open', m):
+            q._test_and_fail_without_cgroupv2()
+            self.assertEqual(0, len(mock_controller.abort.mock_calls))
+
+        m = mock.mock_open(read_data=(
+            'sysfs /sys sysfs '
+            'rw,nosuid,nodev,noexec,relatime 0 0\n'
+            'proc /proc proc '
+            'rw,nosuid,nodev,noexec,relatime 0 0\n'
+            'udev /dev devtmpfs '
+            'rw,nosuid,relatime,size=16029572k,mode=755,inode64 0 0\n'
+            'devpts /dev/pts devpts '
+            'rw,nosuid,noexec,relatime,gid=5,mode=620,ptmxmode=000 0 0\n'
+            'tmpfs /run tmpfs '
+            'rw,nosuid,nodev,noexec,relatime,mode=755,inode64 0 0\n'
+            'securityfs /sys/kernel/security securityfs '
+            'rw,nosuid,nodev,noexec,relatime 0 0\n'
+            'cgroup2 /sys/fs/cgroup/unified cgroup2 '
+            'rw,nosuid,nodev,noexec,relatime,nsdelegate 0 0\n'
+            'cgroup /sys/fs/cgroup/systemd cgroup '
+            'rw,nosuid,nodev,noexec,relatime,xattr,name=systemd 0 0\n'
+            'cgroup /sys/fs/cgroup/net_cls,net_prio cgroup '
+            'rw,nosuid,nodev,noexec,relatime,net_cls,net_prio 0 0\n'
+            'cgroup /sys/fs/cgroup/cpu,cpuacct cgroup '
+            'rw,nosuid,nodev,noexec,relatime,cpu,cpuacct 0 0\n'
+            'cgroup /sys/fs/cgroup/rdma cgroup '
+            'rw,nosuid,nodev,noexec,relatime,rdma 0 0\n'
+            'cgroup /sys/fs/cgroup/freezer cgroup '
+            'rw,nosuid,nodev,noexec,relatime,freezer 0 0\n'
+            'cgroup /sys/fs/cgroup/misc cgroup '
+            'rw,nosuid,nodev,noexec,relatime,misc 0 0\n'
+            'cgroup /sys/fs/cgroup/cpuset cgroup '
+            'rw,nosuid,nodev,noexec,relatime,cpuset 0 0\n'
+            'cgroup /sys/fs/cgroup/devices cgroup '
+            'rw,nosuid,nodev,noexec,relatime,devices 0 0\n'
+            'cgroup /sys/fs/cgroup/memory cgroup '
+            'rw,nosuid,nodev,noexec,relatime,memory 0 0\n'
+            'cgroup /sys/fs/cgroup/pids cgroup '
+            'rw,nosuid,nodev,noexec,relatime,pids 0 0\n'
+            'cgroup /sys/fs/cgroup/blkio cgroup '
+            'rw,nosuid,nodev,noexec,relatime,blkio 0 0\n'
+            'cgroup /sys/fs/cgroup/perf_event cgroup '
+            'rw,nosuid,nodev,noexec,relatime,perf_event 0 0\n'
+            'cgroup /sys/fs/cgroup/hugetlb cgroup '
+            'rw,nosuid,nodev,noexec,relatime,hugetlb 0 0\n'
+            'tmpfs /dev/shm tmpfs '
+            'rw,nosuid,nodev,inode64 0 0\n'
+            'tmpfs /run/lock tmpfs '
+            'rw,nosuid,nodev,noexec,relatime,size=5120k,inode64 0 0\n'
+            'pstore /sys/fs/pstore pstore '
+            'rw,nosuid,nodev,noexec,relatime 0 0\n'
+        ))
+        with mock.patch('DistUpgrade.DistUpgradeQuirks.open', m):
+            q._test_and_fail_without_cgroupv2()
+            self.assertEqual(1, len(mock_controller.abort.mock_calls))
+
 
 class TestSnapQuirks(unittest.TestCase):
 
@@ -644,7 +767,7 @@ class TestSnapQuirks(unittest.TestCase):
                 'channel': 'stable',
                 'command': 'install',
                 'deb': None, 'snap-id': '1234'},
-             'gnome-46-2404': {
+             'gnome-42-2204': {
                 'channel': 'stable/ubuntu-24.10',
                 'command': 'install',
                 'deb': None, 'snap-id': '1234'},
